/*
 * pll_dq_generador_monofasico.c
 *
 *  Created on: 8/3/2016
 *  Author: Carlos
 *
 *  RESUMEN DEL PROYECTO:
 *  Hardware: Conectar una seal senoidal del generador de funciones (50 hz - voltaje pico a pico de 0 a 2.97 V) en el pin6
 *  del J5 de la placa kit piccolo 28027 que corresponde al ADCINA0 del DSP. La seal de ingreso del PLL se vera en buffer y la seal
 *  de salida en buffer_salida. buffer y buffer_salida deben estar en IQ22.
 *  Software: El ADC convierte de 0 a 4096. El cdigo pasa la seal de ingreso a a una seal de voltaje pico a pico de -1V a 1V.
 *  Se aplica el metdo en cuadratura muestreando la seal de ingreso a  12800 muestras en un segundo , Al dividir 12800 hz
 *  entre 50 hz obtenemos 256 muestras por cada seal senoidal de ingreso.
 *
 */

#include "DSP2802x_Device.h"     // DSP2802x Headerfile Include File
#include "pireg.h"
#include "GLOBAL_FUNCS.h"
#include "GLOBAL_VARS.h"
#include <stdio.h>
#include <math.h>
#include "IQmathLib.h"

extern void InitAdc(void);
extern void ServiceDog(void);

void cal_Vpico(void);
void setup_adc(void);
int i=0;
int i_90;
//float salida, ingreso, desfase;
const _iq wff = _IQ(314.1592);	//valor mximo de salida del PI: 2*pi*50= 314.1592 rad/s
const _iq pi_2 = _IQ(1.57078);
//refe,fdb,err,kp,ki_ts"perio-muest",out maxima PI=2*pi*50=314.1592, out minima PI, salida,integral
PIREG compensador = {_IQ(0), 0, 0, _IQ(2), _IQ(7.8E-5), _IQ(314.1592), _IQ(-314.1592), 0, 0,  pireg_compute};
_iq integrador =_IQ(0),  Vsin, Vsin_90, Va=_IQ(0), Vb=_IQ(0), Vpico=_IQ(0),  buffer[255], buffer_salida[255];

main ( ) {

DeviceInit();	// Inicializacin del dispositivo
InitDSP();		// Inicializacion de perifericos
setup_adc();

//	BUCLE
	for(;;)
	{
		asm("          NOP");

	}
}





//	INTERRUPCIN TIMER 2 CADA 78.125us
interrupt void TC0_Cpu_Timer2_isr(void) {

	Va =  _IQmpyI32(_IQ(0.000488281),((long)AdcResult.ADCRESULT0 - 2048));	//ADCRESULT de 0 a 4096, 1V/2048=0.000488281, seal de -1 voltio a 1 voltio
	buffer[i]=Va;
	cal_Vpico();

	i_90 = i + 192;		//256-64=192
	i_90 = (i_90 > 255) ? (i_90 - 256) : (i_90); //operador ternario: variable=condicin?verdadero:falso
	Vb = buffer[i_90];

	//integrador
	Vsin = _IQsin(integrador);
	Vsin_90 = _IQsin(integrador - pi_2);
	buffer_salida[i] = _IQcos(integrador);
	compensador.Fdb = _IQmpy(Va,Vsin) + _IQmpy(Vb,Vsin_90);
	compensador.compute(&compensador);
	integrador += _IQmpy((compensador.Out + wff),_IQ(7.8125E-5));		//7.8125E-5=periodo de muestreo(12800 hz)

	i = (i==255) ? 0 : (i+1);
	////////////////////////////////////////////
	CpuTimer2Regs.TCR.bit.TIF = 1;	// clear flag
	VTimer2++;

}

void cal_Vpico()
{

	Vpico=(Va>Vpico)?Va:Vpico;

}

// 	INTERRUPCIN TIMER 1 CADA 1ms
interrupt void TB0_Cpu_Timer1_isr(void){

	CpuTimer1Regs.TCR.bit.TIF = 1;	// Borrar Flag
	VTimer1++;

}

// 	INTERRUPCIN TIMER 0 CADA 100ms
interrupt void TA0_Cpu_Timer0_isr(void){


	VTimer0++;
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}

void InitDSP(){

	// Deshabilitamos las interrupciones de la CPU
   	DINT;
	// Deshabilitamos las interrupciones de la CPU y borramos los flags
   	IER = 0x0000;
   	IFR = 0x0000;

	InitTimers();	// Inicializacin de los temporizadores

	InitAdc();	// Inicializacin del conversor analgico - digital
	//InitPWM();	// Inicializacion del mdulo PWM 1 y 2

	//InitSci();	// Inicializamos el modulo SCI

	InitInt();	// Inicializacin de las interrupciones
}

void InitTimers (void){

	EALLOW;
		SysCtrlRegs.PCLKCR3.bit.CPUTIMER0ENCLK = 1; // Habilita el reloj del  Timer-0 de la CPU
   		SysCtrlRegs.PCLKCR3.bit.CPUTIMER1ENCLK = 1; // Habilita el reloj del  Timer-1 de la CPU
   		SysCtrlRegs.PCLKCR3.bit.CPUTIMER2ENCLK = 1; // Habilita el reloj del  Timer-2 de la CPU
   	EDIS;

	// Asignamos los manejadores de interrupcin a los timers
	EALLOW;
	   	PieVectTable.TINT0 = &TA0_Cpu_Timer0_isr;
		PieVectTable.TINT1 = &TB0_Cpu_Timer1_isr;
		PieVectTable.TINT2 = &TC0_Cpu_Timer2_isr;
	EDIS;

	// Inicializamos los perifericos de los temporizadores

   	InitCpuTimers();   // For this example, only initialize the Cpu Timers

	// Configuramos los temporiuzadores
	ConfigCpuTimer(&CpuTimer0, 60, 100000);		//100ms
	ConfigCpuTimer(&CpuTimer1, 60, 1000);		//1ms
	ConfigCpuTimer(&CpuTimer2, 60, 78.125);		//desborda cada 78,125 us (12800 hz, 12800 muestras por segundo.),parametro1:frecuen cpu,parametro2:desbor(us)


  	CpuTimer0Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0, Timmer0: run, Habilitado interrupcin por timmer0
   	CpuTimer1Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0, Timmer1: run, Habilitado interrupcin por timmer1
   	CpuTimer2Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0, Timmer2: run, Habilitado interrupcin por timmer2


	/*************************************************************************************/

	VTimer0 = 0;	// Contador del Evento del Timer-0
	VTimer1 = 0; 	// Contador del Evento del Timer-1
	VTimer2 = 0; 	// Contador del Evento del Timer-2
}

void InitInt(void){

	// Configuracin de la interrupcin externa

	/*EALLOW;

		GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 0;         // GPIO-18
		GpioCtrlRegs.GPADIR.bit.GPIO18 = 0;
		GpioCtrlRegs.GPAQSEL2.bit.GPIO18 = 2;
		GpioCtrlRegs.GPAPUD.bit.GPIO18 = 0;
		GpioCtrlRegs.GPACTRL.bit.QUALPRD2 = 0x2F;

	  	GpioCtrlRegs.GPAMUX1.bit.GPIO12 = 0;         // GPIO-12
	   	GpioCtrlRegs.GPADIR.bit.GPIO12 = 0;
	   	GpioCtrlRegs.GPAQSEL1.bit.GPIO12 = 2;
	   	GpioCtrlRegs.GPAPUD.bit.GPIO12 = 1; 	// Para pruebas como se ha utilizado el pulsador, se deshabilita la resistencia de PU
	   	GpioCtrlRegs.GPACTRL.bit.QUALPRD1 = 0x2F;

	   	GpioIntRegs.GPIOXINT1SEL.bit.GPIOSEL = 18;   // XINT1 es GPIO18
		//GpioIntRegs.GPIOXINT1SEL.bit.GPIOSEL = 12;   // XINT1 es GPIO12

	EDIS;
	XIntruptRegs.XINT1CR.bit.POLARITY = 1;      // Flanco SUBIDA de XINT1
	XIntruptRegs.XINT1CR.bit.ENABLE = 1;        // Enable Xint1

	EALLOW;
		PieVectTable.XINT1 = &xint1_isr;	// Interrupcion XINT1
	EDIS;*/
	// Enable Peripheral, global Ints and higher priority real-time debug events:
   	IER |= (M_INT1|M_INT2);
   	IER |= (M_INT3|M_INT4);

   	IER |= M_INT8;
   	IER |= M_INT9; // Esta deja Bloqueado al HW...

	// Activar interrupcin XINT1
	//PieCtrlRegs.PIEIER1.bit.INTx4 = 1;   // XINT1

	// Activar interrupcin sci fifo rx
	//PieCtrlRegs.PIEIER9.bit.INTx1 = 1;   // RX SCIA

	// Activar interrupciones ePWM
	//PieCtrlRegs.PIEIER3.bit.INTx1 = 1;	//	ePWM1
	//PieCtrlRegs.PIEIER3.bit.INTx2 = 1;	//	ePWM2

	// Activar Interrupciones Timers-CPU
	PieCtrlRegs.PIEIER1.bit.INTx7 = 1; 	//	T0
   	IER |= (M_INT13 |M_INT14) ;			//	T1 y T2
   	//////////////////////////////////////////////////////////
	// Enable the PIE
    PieCtrlRegs.PIECTRL.bit.ENPIE = 1;

	// Enables PIE to drive a pulse into the CPU
	PieCtrlRegs.PIEACK.all = 0xFFFF;

	// Enable Interrupts at the CPU level
    EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM
	//////////////////////////////////////////////////////

}

void setup_adc(void){

	// Configure ADC
		EALLOW;
		AdcRegs.ADCCTL1.bit.INTPULSEPOS	= 1;	//ADCINT1 trips after AdcResults latch
		AdcRegs.INTSEL1N2.bit.INT1E     = 1;	//habilita interrupcin ADC1
		AdcRegs.INTSEL1N2.bit.INT1CONT  = 0;	//Habilita ADCINT1 Continuous mode
		AdcRegs.INTSEL1N2.bit.INT1SEL	= 1;	//setup EOC1 to trigger ADCINT1 to fire
		AdcRegs.ADCSOC0CTL.bit.CHSEL 	= 0;	//set SOC0 channel select to ADCINA0
		//AdcRegs.ADCSOC1CTL.bit.CHSEL 	= 1;	//set SOC1 channel select to ADCINA1
		//AdcRegs.ADCSOC2CTL.bit.CHSEL 	= 2;	//set SOC1 channel select to ADCINA2
		//AdcRegs.ADCSOC3CTL.bit.CHSEL 	= 3;	//set SOC1 channel select to ADCINA3
		AdcRegs.ADCSOC0CTL.bit.TRIGSEL 	= 3;	//empieza con el desbordamiento del timer2
		//AdcRegs.ADCSOC1CTL.bit.TRIGSEL 	= 5;	//set SOC1 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1
		//AdcRegs.ADCSOC2CTL.bit.TRIGSEL 	= 5;	//set SOC0 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1
		//AdcRegs.ADCSOC3CTL.bit.TRIGSEL 	= 5;	//set SOC1 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1
		AdcRegs.ADCSOC0CTL.bit.ACQPS 	= 6;	//set SOC0 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
		//AdcRegs.ADCSOC1CTL.bit.ACQPS 	= 6;	//set SOC1 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
		//AdcRegs.ADCSOC2CTL.bit.ACQPS 	= 6;	//set SOC0 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
		//AdcRegs.ADCSOC3CTL.bit.ACQPS 	= 6;	//set SOC1 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
		EDIS;

	// Assumes ePWM1 clock is already enabled in InitSysCtrl();
	//EPwm1Regs.ETSEL.bit.SOCAEN	= 1;		// Enable SOC on A group
	//EPwm1Regs.ETSEL.bit.SOCASEL	= ET_CTR_PRDZERO;		// Select SOC from from CPMA on upcount
	//EPwm1Regs.ETPS.bit.SOCAPRD 	= ET_1ST;		// Generate pulse on 1st event


}
